"""Constants used by vizio component."""

CONF_VOLUME_STEP = "volume_step"

DEFAULT_NAME = "Vizio SmartCast"
DEFAULT_VOLUME_STEP = 1
DEFAULT_DEVICE_CLASS = "tv"
DEVICE_ID = "pyvizio"
DEVICE_NAME = "Python Vizio"

DOMAIN = "vizio"

ICON = {"tv": "mdi:television", "soundbar": "mdi:speaker"}
